@@ -36,14 +36,13 @@ use crate::analytics::sync::send_analytics_event;
 use crate::settings::Settings;
 use crate::shards::errors::ShardNotFoundError;
 use crate::shards::metadata::ShardMetadata;
-use crate::shards::providers::unbounded_cache::AsyncUnboundedShardWriterCache;
-use crate::shards::providers::AsyncShardWriterProvider;
+use crate::shards::providers::shard_cache::ShardWriterCache;
 use crate::shards::writer::ShardWriter;
 use crate::telemetry::run_with_telemetry;
 use crate::utils::list_shards;
 
 pub struct NodeWriterGRPCDriver {
-    shards: Arc<AsyncUnboundedShardWriterCache>,
+    shards: Arc<ShardWriterCache>,
     sender: Option<UnboundedSender<NodeWriterEvent>>,
     settings: Settings,
 }
@@ -55,7 +54,7 @@ pub enum NodeWriterEvent {
 }
 
 impl NodeWriterGRPCDriver {
-    pub fn new(settings: Settings, shard_cache: Arc<AsyncUnboundedShardWriterCache>) -> Self {
+    pub fn new(settings: Settings, shard_cache: Arc<ShardWriterCache>) -> Self {
         Self {
             settings,
             shards: shard_cache,
@@ -63,15 +62,6 @@ impl NodeWriterGRPCDriver {
         }
     }
 
-    /// This function must be called before using this service
-    pub async fn initialize(&self) -> NodeResult<()> {
-        if !self.settings.lazy_loading() {
-            // If lazy loading is disabled, load
-            self.shards.load_all().await?
-        }
-        Ok(())
-    }
-
     pub fn with_sender(self, sender: UnboundedSender<NodeWriterEvent>) -> Self {
         Self {
             sender: Some(sender),
@@ -79,21 +69,6 @@ impl NodeWriterGRPCDriver {
         }
     }
 
-    async fn obtain_shard(&self, id: impl Into<String>) -> Result<Arc<ShardWriter>, tonic::Status> {
-        let id = id.into();
-        if let Some(shard) = self.shards.get(id.clone()).await {
-            return Ok(shard);
-        }
-        let shard = self.shards.load(id.clone()).await.map_err(|error| {
-            if error.is::<ShardNotFoundError>() {
-                tonic::Status::not_found(error.to_string())
-            } else {
-                tonic::Status::internal(format!("Error lazy loading shard {id}: {error:?}"))
-            }
-        })?;
-        Ok(shard)
-    }
-
     #[tracing::instrument(skip_all)]
     fn emit_event(&self, event: NodeWriterEvent) {
         if let Some(sender) = &self.sender {
@@ -102,6 +77,22 @@ impl NodeWriterGRPCDriver {
     }
 }
 
+fn obtain_shard(
+    shards: Arc<ShardWriterCache>,
+    id: impl Into<String>,
+) -> Result<Arc<ShardWriter>, tonic::Status> {
+    let id = id.into();
+
+    let shard = shards.get(&id).map_err(|error| {
+        if error.is::<ShardNotFoundError>() {
+            tonic::Status::not_found(error.to_string())
+        } else {
+            tonic::Status::internal(format!("Error lazy loading shard {id}: {error:?}"))
+        }
+    })?;
+    Ok(shard)
+}
+
 #[tonic::async_trait]
 impl NodeWriter for NodeWriterGRPCDriver {
     async fn new_shard(
@@ -119,7 +110,12 @@ impl NodeWriter for NodeWriterGRPCDriver {
             request.similarity().into(),
             Some(Channel::from(request.release_channel)),
         );
-        let new_shard = self.shards.create(metadata).await;
+
+        let shards = Arc::clone(&self.shards);
+        let new_shard = tokio::task::spawn_blocking(move || shards.create(metadata))
+            .await
+            .map_err(|error| tonic::Status::internal(format!("Error creating shard: {error:?}")))?;
+
         match new_shard {
             Ok(new_shard) => {
                 self.emit_event(NodeWriterEvent::ShardCreation);
@@ -139,7 +135,15 @@ impl NodeWriter for NodeWriterGRPCDriver {
         send_analytics_event(AnalyticsEvent::Delete).await;
         // Deletion does not require for the shard to be loaded.
         let shard_id = request.into_inner();
-        let deleted = self.shards.delete(shard_id.id.clone()).await;
+
+        let shards = Arc::clone(&self.shards);
+        let shard_id_clone = shard_id.id.clone();
+        let deleted = tokio::task::spawn_blocking(move || shards.delete(&shard_id_clone))
+            .await
+            .map_err(|error| {
+                tonic::Status::internal(format!("Error deleted shard {}: {error:?}", shard_id.id))
+            })?;
+
         match deleted {
             Ok(_) => {
                 self.emit_event(NodeWriterEvent::ShardDeletion);
@@ -158,8 +162,17 @@ impl NodeWriter for NodeWriterGRPCDriver {
         request: Request<ShardId>,
     ) -> Result<Response<ShardCleaned>, Status> {
         let shard_id = request.into_inner().id;
+
         // No need to load shard to upgrade it
-        match self.shards.upgrade(shard_id).await {
+        let shards = Arc::clone(&self.shards);
+        let shard_id_clone = shard_id.clone();
+        let upgraded = tokio::task::spawn_blocking(move || shards.upgrade(&shard_id_clone))
+            .await
+            .map_err(|error| {
+                tonic::Status::internal(format!("Error deleted shard {}: {error:?}", shard_id))
+            })?;
+
+        match upgraded {
             Ok(upgrade_details) => Ok(tonic::Response::new(upgrade_details)),
             Err(error) => Err(tonic::Status::internal(error.to_string())),
         }
@@ -183,10 +196,12 @@ impl NodeWriter for NodeWriterGRPCDriver {
         let span = Span::current();
         let resource = request.into_inner();
         let shard_id = resource.shard_id.clone();
-        let shard = self.obtain_shard(&shard_id).await?;
+        let shards = Arc::clone(&self.shards);
+        let shard_id_clone = shard_id.clone();
         let info = info_span!(parent: &span, "set resource");
         let write_task = || {
             run_with_telemetry(info, move || {
+                let shard = obtain_shard(shards, shard_id_clone)?;
                 shard
                     .set_resource(&resource)
                     .and_then(|()| shard.get_opstatus())
@@ -223,10 +238,12 @@ impl NodeWriter for NodeWriterGRPCDriver {
         let span = Span::current();
         let resource = request.into_inner();
         let shard_id = resource.shard_id.clone();
-        let shard = self.obtain_shard(&shard_id).await?;
+        let shards = Arc::clone(&self.shards);
+        let shard_id_clone = shard_id.clone();
         let info = info_span!(parent: &span, "remove resource");
         let write_task = || {
             run_with_telemetry(info, move || {
+                let shard = obtain_shard(shards, shard_id_clone)?;
                 shard
                     .remove_resource(&resource)
                     .and_then(|()| shard.get_opstatus())
@@ -275,10 +292,12 @@ impl NodeWriter for NodeWriterGRPCDriver {
             None => return Err(tonic::Status::invalid_argument("Shard ID must be provided")),
         };
 
-        let shard = self.obtain_shard(shard_id).await?;
+        let shards = Arc::clone(&self.shards);
+        let shard_id_clone = shard_id.clone();
         let info = info_span!(parent: &span, "add vector set");
         let task = || {
             run_with_telemetry(info, move || {
+                let shard = obtain_shard(shards, shard_id_clone)?;
                 shard
                     .add_vectorset(&vectorset_id, request.similarity())
                     .and_then(|()| shard.get_opstatus())
@@ -307,10 +326,12 @@ impl NodeWriter for NodeWriterGRPCDriver {
             Some(ref shard_id) => &shard_id.id,
             None => return Err(tonic::Status::invalid_argument("Shard ID must be provided")),
         };
-        let shard = self.obtain_shard(shard_id).await?;
+        let shards = Arc::clone(&self.shards);
+        let shard_id_clone = shard_id.clone();
         let info = info_span!(parent: &span, "remove vector set");
         let task = || {
             run_with_telemetry(info, move || {
+                let shard = obtain_shard(shards, shard_id_clone)?;
                 shard
                     .remove_vectorset(&request)
                     .and_then(|()| shard.get_opstatus())
@@ -335,9 +356,13 @@ impl NodeWriter for NodeWriterGRPCDriver {
     ) -> Result<Response<VectorSetList>, Status> {
         let span = Span::current();
         let shard_id = request.into_inner();
-        let shard = self.obtain_shard(shard_id.id.clone()).await?;
+        let shards = Arc::clone(&self.shards);
+        let shard_id_clone = shard_id.id.clone();
         let info = info_span!(parent: &span, "list vector sets");
-        let task = || run_with_telemetry(info, move || shard.list_vectorsets());
+        let task = || {
+            let shard = obtain_shard(shards, shard_id_clone)?;
+            run_with_telemetry(info, move || shard.list_vectorsets())
+        };
         let status = tokio::task::spawn_blocking(task).await.map_err(|error| {
             tonic::Status::internal(format!("Blocking task panicked: {error:?}"))
         })?;
@@ -371,10 +396,14 @@ impl NodeWriter for NodeWriterGRPCDriver {
     ) -> Result<Response<GarbageCollectorResponse>, Status> {
         send_analytics_event(AnalyticsEvent::GarbageCollect).await;
         let shard_id = request.into_inner();
-        let shard = self.obtain_shard(&shard_id.id).await?;
+        let shards = Arc::clone(&self.shards);
+        let shard_id_clone = shard_id.id.clone();
         let span = Span::current();
         let info = info_span!(parent: &span, "list vector sets");
-        let task = || run_with_telemetry(info, move || shard.gc());
+        let task = || {
+            let shard = obtain_shard(shards, shard_id_clone)?;
+            run_with_telemetry(info, move || shard.gc())
+        };
         let result = tokio::task::spawn_blocking(task).await.map_err(|error| {
             tonic::Status::internal(format!("Blocking task panicked: {error:?}"))
         })?;
