@@ -22,7 +22,7 @@ use std::path::{Path, PathBuf};
 use std::sync::Arc;
 
 use nucliadb_core::tracing::{debug, error, info, warn};
-use nucliadb_core::NodeResult;
+use nucliadb_core::{node_error, NodeResult};
 use nucliadb_protos::{noderesources, replication};
 use tokio::fs::File;
 use tokio::io::AsyncReadExt;
@@ -32,35 +32,23 @@ use tonic::Response;
 use crate::replication::NodeRole;
 use crate::settings::Settings;
 use crate::shards::metadata::Similarity;
-use crate::shards::providers::unbounded_cache::AsyncUnboundedShardWriterCache;
-use crate::shards::providers::AsyncShardWriterProvider;
+use crate::shards::providers::shard_cache::ShardWriterCache;
 use crate::shards::writer::ShardWriter;
 use crate::utils::list_shards;
 pub struct ReplicationServiceGRPCDriver {
     settings: Settings,
-    shards: Arc<AsyncUnboundedShardWriterCache>,
+    shards: Arc<ShardWriterCache>,
     node_id: String,
 }
 
 impl ReplicationServiceGRPCDriver {
-    pub fn new(
-        settings: Settings,
-        shard_cache: Arc<AsyncUnboundedShardWriterCache>,
-        node_id: String,
-    ) -> Self {
+    pub fn new(settings: Settings, shard_cache: Arc<ShardWriterCache>, node_id: String) -> Self {
         Self {
             settings,
             shards: shard_cache,
             node_id,
         }
     }
-
-    /// This function must be called before using this service
-    pub async fn initialize(&self) -> NodeResult<()> {
-        // should we do this?
-        self.shards.load_all().await?;
-        Ok(())
-    }
 }
 
 async fn stream_file(
@@ -73,11 +61,9 @@ async fn stream_file(
     let filepath = shard_path.join(rel_filepath);
 
     if !filepath.exists() {
-        debug!(
-            "File not found when index thought it should be: {}",
-            filepath.to_string_lossy(),
-        );
-        return Ok(());
+        return Err(node_error!(
+            "This file can not be streamed because it does not exist"
+        ));
     }
 
     debug!("Streaming file {}", filepath.to_string_lossy());
@@ -256,11 +242,19 @@ impl replication::replication_service_server::ReplicationService for Replication
             Result<replication::ReplicateShardResponse, tonic::Status>,
         > = receiver.0.clone();
 
-        let shard_lookup = self.shards.load(request.shard_id.clone()).await;
+        let id = request.shard_id;
+        let id_clone = id.clone();
+        let shards = self.shards.clone();
+        let shard_lookup = tokio::task::spawn_blocking(move || shards.get(&id_clone))
+            .await
+            .map_err(|error| {
+                tonic::Status::internal(format!("Error lazy loading shard {id}: {error:?}"))
+            })?;
+
         if let Err(error) = shard_lookup {
             return Err(tonic::Status::not_found(format!(
                 "Shard {} not found, error: {}",
-                request.shard_id, error
+                id, error
             )));
         }
 
